function [all_ensinfo, last_ensinfo, newblock] = ...
    get_next_ensinfo(rawdir, rawbaselist, last_ensinfo, params, newdb)
% [all_ensinfo, last_ensinfo, newblock] = ...
%                               get_next_ensinfo(rawdir, rawbaselist, last_ensinfo, params)
% ensinfo is a structure array with one entry per call to read, fieldnames:
% rawfilebase, index, pingtype, ensnums, 


if isempty(rawbaselist)
   ensinfo = struct;
   return
end


%% take care of the case where the database is just starting
if newdb == 1
   %we are starting a new database.  don't have pingtypes yet
   %pretend we got pingtypes from ifile == 0
   last_ensinfo.pingtypes = ...
       get_pingtypes(params.instclass, rawdir, rawbaselist{1}, params);
end

%%%  set some variables
% use this as a criterion for being "close enough" to accept an ensemble
close_enough = max(20, .1*params.interval);
have_an_ensemble    = 0;
all_ensinfo         = [];
interval_remaining  = params.interval;  % in seconds!!
last_ensnum         = last_ensinfo.lastens;
newblock            = 0;


while (have_an_ensemble == 0)
   
   %% initialize structure so get_next_piece will return ensinfo with 
   %% the right order

   % failure to get data from current file or new configuration
   if newdb == 1, 
      fprintf('about to read first ens in new database\n'), 
      newblock                  = 1;
      last_ensinfo.ifile        = last_ensinfo.ifile +1;
      last_ensinfo.lastens      = 0;
      last_ensinfo.firstens     = 10^10;
      last_ensinfo.rawfilebase  = rawbaselist{last_ensinfo.ifile};

      last_ensinfo.pingtypes = ...
          get_pingtypes(params.instclass, rawdir, last_ensinfo.rawfilebase, params);
      
      %reset values to start get_another_piece for a whole ensemble
      all_ensinfo = [];
      interval_remaining = params.interval;
      newdb = 0;
      last_ensnum = 0;
      continue
   else   
      % attempt to get up to 'interval' worth from a new file
      % ensinfo includes only: ensnums, ilist
      [ensinfo, interval_read] = ...
          get_another_piece(rawdir, last_ensinfo, interval_remaining, params, close_enough);
   end
   if interval_read ~= 0
      fprintf('got %d seconds of data\n', interval_read)
   end
   
   % nothing to read (may not be a problem) try again
   if length(fieldnames(ensinfo)) == 0 
      fprintf('attempt to read ens segment failed\n')
      if last_ensinfo.ifile == length(rawbaselist)
         % out of data: tried to read from the last file
         fprintf('no more data in this file list\n')
         last_ensinfo = struct;
         newblock = -1;
         all_ensinfo = [];
         return
      else
         %try the next file
         fprintf('nothing left in this file: move on to the next file\n')
         newblock                  = 0;
         last_ensinfo.ifile        = last_ensinfo.ifile +1;
         last_ensinfo.rawfilebase  = rawbaselist{last_ensinfo.ifile}; %new number
         
         %% take a peek to see whether the next file is a continuation
         nextrawlogfile = fullfile(rawdir, [last_ensinfo.rawfilebase, '.raw.log']);
         fprintf('peeking at raw log file:  %s\n', nextrawlogfile)
         [rlog_udday, rlog_offsets, rlog_bytes, rlog_ensnums, rlog_instdday] = ...
             textread( nextrawlogfile, '%f%d%d%d%f');
         if rlog_ensnums(1)  == 1
            last_ensinfo.lastens = 0
            last_ensinfo.firstens = 10^10
            interval_remaining = params.interval;
         else
            interval_remaining = interval_remaining - interval_read;
         end
         continue
      end
   else
      % if we made it here, we have some data.
      if  ensinfo.firstens == 1
         fprintf('starting at the beginning of a new sequence of files\n')
         %  starting a new block
         fprintf('read data from a new file\n')
         fprintf('get_next_ensinfo resetting newblock=1\n')
         if interval_read > close_enough
            have_an_ensemble = 1;
            clear last_ensinfo
            %re-initialize for printing and next go-around
            last_ensinfo.lastens      = ensinfo.lastens;
            last_ensinfo.ifile        = ensinfo.ifile;
            last_ensinfo.rawfilebase  = ensinfo.rawfilebase;
            last_ensinfo.pingtypes    = ensinfo.pingtypes;
            newblock                  = 1;
            all_ensinfo               = ensinfo;
            break
         else
            newblock                  = 1;
            last_ensinfo.lastens      = 0;
            last_ensinfo.firstens     = 10^10;
            last_ensinfo.pingtypes = ...
                get_pingtypes(params.instclass, rawdir, last_ensinfo.rawfilebase, params);
            %reset values to start get_another_piece for a whole ensemble
            all_ensinfo = [];
            interval_remaining = params.interval;
            newdb = 0;
            last_ensnum = 0;
         end
      else
         % good data from the middle
         lastens_ensnum  =  ensinfo.lastens;
         interval_remaining = interval_remaining - interval_read;
         
         if interval_remaining < close_enough
            have_an_ensemble = 1;
         end
         all_ensinfo = [all_ensinfo ; ensinfo];
         
         clear last_ensinfo
         %re-initialize for printing and next go-around
         
         last_ensinfo.lastens      = ensinfo.lastens;
         last_ensinfo.ifile        = ensinfo.ifile;
         last_ensinfo.rawfilebase  = ensinfo.rawfilebase;
         last_ensinfo.pingtypes    = ensinfo.pingtypes;
      end
   end

end
   
%--------------------------------------------

function [ensinfo, interval_read] = ...
    get_another_piece(rawdir, last_ensinfo,  interval2read, params, close_enough);

% 
% ensinfo starts with fields rawfilebase, rawdir
% we add ensnums,
% return with empty structure file is too short or pingtype is wrong

% this is useful to has as a variable if ensinfo fails
interval_read = 0;

rawlogfile = fullfile(rawdir, [last_ensinfo.rawfilebase, '.raw.log']);

% get raw logfile info
[rlog_udday, rlog_offsets, rlog_bytes, rlog_ensnums, rlog_instdday] = ...
    textread( rawlogfile, '%f%d%d%d%f');

% just skip it if it is one ensemble long
if length(rlog_udday) ==1
   ensinfo = struct;
   return
end

%% if the whole file starts with 1 and is too short SKIP  IT
if rlog_ensnums(1) == 1 & ...
        round(86400*diff(rlog_instdday([1 end]))) < close_enough
   fprintf('file %s too short\n', rawlogfile)
   ensinfo = struct;
   return
end

if rlog_ensnums(1) < 5
   fprintf('reading raw log file:  %s\n', rawlogfile)
end

% profiles are sometimes skipped
istart = min(find(rlog_ensnums >= (last_ensinfo.lastens + 1)));


if isempty(istart)
   ensinfo = struct;
   return
end
   
  
ens_inst_startdday = rlog_instdday(istart);
ens_inst_enddday =  rlog_instdday(istart) + interval2read/86400;

index = find( (rlog_instdday >= ens_inst_startdday) & ...
              (rlog_instdday <= ens_inst_enddday));

if isempty(index)
   ensinfo = struct;
   fprintf('there is a problem with raw logfile indices in %s\n', mfilename)
   return
end

interval_read = round(86400*diff(rlog_instdday(index([1 end]))));

% update some things we've learned
ensinfo.firstens = rlog_ensnums(index(1));
ensinfo.lastens  = rlog_ensnums(index(end));
ensinfo.ilist = index;
% transfer some things from last_ensinfo
ensinfo.ifile       = last_ensinfo.ifile;
ensinfo.rawfilebase = last_ensinfo.rawfilebase;
ensinfo.pingtypes   = last_ensinfo.pingtypes;
ensinfo.secs_read   = interval_read;





%% --- end of get_another_piece -----------------------------------